
public final class VersionNumber implements Comparable {

    private final int release;
    private final int revision;
    private final int patch;

    public VersionNumber(int release, int revision, int patch) {
        this.release  = release;
        this.revision = revision;
        this.patch    = patch;
    }

    public String toString() {
        return "(" + release + "." + revision + "." + patch + ")";
    }

    public boolean equals(Object obj) {                 // (1)
        if (obj == this)                                // (2)
            return true;
        if (!(obj instanceof VersionNumber))            // (3)
            return false;
        VersionNumber vno = (VersionNumber) obj;        // (4)
        return vno.patch    == this.patch    &&         // (5)
               vno.revision == this.revision &&
               vno.release  == this.release;
    }

    public int hashCode() {                             // (6)
        int hashValue = 11;
        hashValue = 31 * hashValue + release;
        hashValue = 31 * hashValue + revision;
        hashValue = 31 * hashValue + patch;
        return hashValue;
    }

    public int compareTo(Object obj) {                  // (7)
        VersionNumber vno = (VersionNumber) obj;        // (8)

        // Compare the release numbers.                    (9)
        if (release < vno.release)
            return -1;
        if (release > vno.release)
            return 1;

        // Release numbers are equal,                      (10)
        // must compare revision numbers.
        if (revision < vno.revision)
            return -1;
        if (revision > vno.revision)
            return 1;

        // Release and revision numbers are equal,         (11)
        // must compare patch numbers.
        if (patch < vno.patch)
            return -1;
        if (patch > vno.patch)
            return 1;

        // All fields are equal.                           (12)
        return 0;
    }
}